/** \file
 * Hello world example app.
 * A very simple mpi skeleton application.
 */

#include "helloapp.h"
#include <stdio.h>

using namespace sstmac::sw;

namespace hellocomm {

  // Public constructor
  // Creates a reference counted app object
  helloapp::ptr
  helloapp::construct(softwareid id) {
    helloapp::ptr obj(new helloapp(id));
    return obj;
  }

  // Get a copy 
  // Launchers can use this to create app threads 
  app::ptr
  helloapp::clone(softwareid id) {
    app::ptr obj(new helloapp(id));
    return obj;
  }

  // Overrides app::skeleton_main virtual function
  // The task thread context is started by a call into this function
  void helloapp::skeleton_main() { 

    // initialize mpi
    if(mpi() == 0)
      throw sstmac::nullerror("helloapp:  mpiapi pointer is null.");

    sstmac::timestamp start = mpi()->init();
    sstmac::sw::mpicomm world(mpi()->comm_world());
    sstmac::sw::mpiid rank(world.rank());
    std::cout << "multmpi[" << rank << "]::run(): mpi()->init() at " 
            //  << start
              << "\n";

    // simulate broadcasting 1 double from rank 0 with no real payload
    const int count=1;
    const sstmac::sw::mpiid root(0);
    mpi()->bcast(count, sstmac::sw::mpitype::mpi_double, root, world);
    if(rank == sstmac::sw::mpiid(0))
      std::cout << rank << ": " << message_ << std::endl;
    else
      std::cout << rank << ": " << message_ << std::endl;

    // finalize 
   // sstmac::timestamp done = mpi()->finalize();
    std::cout << "multmpi[" << rank << "]::run():  mpi()->finalize() at " 
             // << done
              << "\n";
  }

  // Overrides app::consum_params virtual function
  void helloapp::consume_params(const std::list<std::string> &params) {
    if (params.size() != 1) {
      throw sstmac::valueerror("helloapp::consume_params: require one param");
    }
    message_ = *params.begin();
  }

} // end of namespace hellosim
