/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_COMPONENTS_NIC_SIMPLENIC_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_COMPONENTS_NIC_SIMPLENIC_H_INCLUDED


#include <sstmac/backends/native/components/nic/networkinterface.h>

namespace sstmac { namespace native
  {

  /**
   * A private nested type to handle sending data.
   */
  class simplenic: public networkinterface
  {



  protected:
    /// Hi.
    simplenic(const node::ptr &parent,
        const sstmac::hw::interconnect::ptr &interconn);

  public:
    typedef boost::intrusive_ptr<simplenic> ptr;

    /// Hello.
    static simplenic::ptr
    construct(const node::ptr &parent,
        const sstmac::hw::interconnect::ptr &interconn);

    /// Goodbye.
    virtual ~simplenic() throw ();

    /// Schedule a send operation to happen at the given time.
    /// \param start_time is the time when the send is started.
    /// \param ppidarg is the parallel process id of the source and target.
    /// \param recver is the global index of the receiver node.
    /// \param payload is the data sent over the network
    /// \param completion is the eventhandler signaled when the send is complete
    virtual void send(const nodeid::ptr &recver,
        const sst_message::ptr &payload,
        const eventhandler::ptr &completion);

    virtual networkinterface::ptr
    clone(const node::ptr &parent,
        const sstmac::hw::interconnect::ptr &interconn) const;

    virtual void handle(const sst_message::ptr &msg);

  };

  } } // end of namespace sstmac.

#endif
