/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_COMPONENTS_NODE_SIMPLENODE_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_COMPONENTS_NODE_SIMPLENODE_H_INCLUDED

#include <sstmac/hardware/processor/simpleprocessor.h>
#include <sstmac/hardware/processor/instructionprocessor.h>
#include <sstmac/hardware/processor/computescheduler.h>
#include <sstmac/backends/native/components/nic/networkinterface.h>
#include <sstmac/backends/native/components/node/node.h>
#include <sstmac/common/nocopy.h>

#include <map>

//#include <sstmac/processor/processor_config.h>

namespace sstmac { namespace native
  {

  // Forward declaration of important types.
  //class procscheduler;

  /**
   * A stand-in node.  This node type passes all requests directly to
   * the simulator and has no resource contention (infinite processors),
   */
  class simplenode: public node, virtual public nocopy
  {

  protected:
    simplenode(const timestamp &memory_latency, double memory_bandwidth,
        int n_core);

    networkinterface::ptr nic_;

    timestamp memory_latency_;
    double memory_bandwidth_;
    int cores_;

    sstmac::hw::simpleprocessor::ptr proc_model_;
    sstmac::hw::instructionprocessor::ptr proc_inst_;
    sstmac::hw::computescheduler::ptr compute_sched_;

  public:
    typedef boost::intrusive_ptr<simplenode> ptr;

    /// Construct a new node with the given memory bandwidth.
    static simplenode::ptr
    construct(const timestamp &memory_latency, double memory_bandwidth,
        int n_core = 1);

    // ------- AMI functions --------- //
    virtual bool
    kernel_supported(int func);

    virtual timestamp
    now();

    virtual void
    execute_kernel(int func, sst_message::ptr data);

    virtual ptr_type::ptr
    execute_kernel(int func);


    // --------------------- //


    virtual node::ptr
    clone() const;

    virtual void
    set_nic(const networkinterface::ptr &n);

    virtual networkinterface::ptr
    get_nic() const;

    virtual void set_eventmanager(const eventmanager::ptr &m);

    virtual void
    initialize(nodeid::ptr my_id, const sstmac::sw::service::ptr &the_launcher);



  };

  } } // end of namespace sstmac

#endif
