/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_EVENTMAP_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_EVENTMAP_H_INCLUDED

#include <sstmac/common/eventmanager.h>
#include <sstmac/backends/native/sstevent.h>
#include <sstmac/common/timestamp.h>
#include <map>

namespace sstmac { namespace native
{

  /**
   * An event manager that relies on a multimap to manage events.
   */
  class eventmap : public eventmanager
  {
    /// The events sorted by timestamp.
    typedef std::multimap<timestamp, sst_event::ptr > queue_t;
    queue_t queue_;

    /// Sentinel to track whether the event handler is running or not.
    bool running_;


    timestamp last_update_sim_;

    /// Hello.
    eventmap(const timestamp &tv);

  public:

    typedef boost::intrusive_ptr<eventmap> ptr;
    /// Hello.
    static eventmap::ptr
    construct(const timestamp &tv = timestamp(0));

    /// Goodbye.
    virtual
    ~eventmap() throw ();

    /// Clear all events and set time back to a zero of your choice.
    /// \throw illformederror if the event manager is running.
    virtual void
    clear(const timestamp &zero_time = timestamp(0));

    /// Run the eventmanager.
    /// The eventmanager shall return control when no more messages remain.
    virtual void
    run();

    /// Set off the given eventhandler at the given time.
    virtual void
    schedule(const timestamp &start_time,
        const eventhandler::ptr &handler,
        const sst_message::ptr &msg);
  };

} } // end of namespace sstmac

#endif
