/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_NODELIST_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_NODELIST_H_INCLUDED

#include <sstmac/backends/native/components/node/node.h>
#include <sstmac/backends/native/nodeid.h>
#include <sstmac/common/ptr_type.h>
#include <vector>

namespace sstmac  { namespace native
  {


  /**
   * A shared mapping from nodeid indices onto node pointers.
   */
  class nodelist: public ptr_type
 {
    /// My nodes.
    std::vector< node::ptr > nodes_;

  protected:
    /// Hi there.
    nodelist();

  public:
    virtual std::string to_string() const{
            return "nodeist";
          }

    typedef boost::intrusive_ptr<nodelist> ptr;

    /// Welcome to the new tomorrow.
    static nodelist::ptr construct();

    /// The number of nodes in the list.
    int64_t size() const;

    /// Get the given node in the list.
    node::ptr get(const nodeid::ptr &id) const;

    /// Add the given number of nodes to the list using the
    /// given object as a template.
    /// \return the index of the new node.
    nodeid::ptr add(const node::ptr &the_node);
  };

} } // end of namespace sstmac

#endif
