/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#include <iomanip>
#include <iostream>

#include <networkmodels/rackswitch.h>

#include <networkmodels/routing/crossbarrouter.h>
#include <networkmodels/routing/cube3drouter.h>
#include <networkmodels/routing/torus3drouter.h>

namespace sstmac { namespace omnetpp {


  using namespace std;

  debug rackswitch::dbg_;

  /*
map<cChannel*, rackswitch*> rackswitch::channel_map1_;
map<cChannel*, rackswitch*> rackswitch::channel_map2_;

map<cChannel*, int> rackswitch::channel_vals1_;
map<cChannel*, int> rackswitch::channel_vals2_;
   */

  rackswitch::~rackswitch()
  {
    delete router_;
  }

  void
  rackswitch::instantiateRouter(std::string top, int p1, int p2, int p3)
  {

    if (top.compare("crossbar") == 0)
      {
        router_ = new crossbarrouter(myAddr, nodesPerRack);
      }
    else if (top.compare("3DCube") == 0)
      {
        router_ = new cube3drouter(myAddr, nodesPerRack, p1, p2, p3);
      }
    else if (top.compare("3DTorus") == 0)
      {
        router_ = new torus3drouter(myAddr, nodesPerRack, p1, p2, p3);
      }
    else
      {
        std::cout << "Topology: " << top << endl;
        opp_error("rackswitch: unknown topology");
      }

  }

  /*
void
rackswitch::register_channel(cChannel* ch, rackswitch* rs)
{
  if (displayConnections)
    {
      if (channel_map1_.find(ch) != channel_map1_.end())
        {
          channel_map2_[ch] = rs;
          channel_vals2_[ch] = 0;
        }
      else
        {
          channel_map1_[ch] = rs;
          channel_vals1_[ch] = 0;
        }
    }
}

void
rackswitch::update_channel_display(cChannel* ch, rackswitch* rs, double frac)
{

  if (displayConnections)
    {
      cDisplayString& connDispStr = ch->getDisplayString();

      if (channel_map1_[ch] == rs)
        {

          std::stringstream ss;
          int linewidth = (1 + frac * 8);

          ss << "ls=#";

          int val = 255 * (frac > 1 ? 1.0 : frac);
          channel_vals1_[ch] = val; //save it so the other guy can use it

          ss << hex << std::setw(2) << channel_vals1_[ch]; //red

          ss << "00"; //green

          ss << hex << std::setw(2) << channel_vals2_[ch]; //blue

          ss << "," << dec << linewidth;

          string dis = ss.str();

          replace(dis.begin(), dis.end(), ' ', '0');


          //cout << "connection display: " << dis;
          connDispStr.parse(dis.c_str());
        }
      else
        {
          std::stringstream ss;
          int linewidth = (1 + frac * 8);

          ss << "ls=#";
          int val = 255 * (frac > 1 ? 1.0 : frac);
          channel_vals2_[ch] = val; //save it so the other guy can use it

          ss << hex << std::setw(2) << channel_vals1_[ch];

          ss << "00"; //green

          ss << hex << std::setw(2) << channel_vals2_[ch];

          ss << "," << dec << linewidth;

          string dis = ss.str();

          replace(dis.begin(), dis.end(), ' ', '0');

          //cout << "connection display: " << dis;
          connDispStr.parse(dis.c_str());
        }
    }
}*/
} }
