/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include "rackallocator.h"

#include <Node/Processor/space/rackspace.h>

namespace sstmac { namespace omnetpp {


  using namespace std;

  rackallocator::rackallocator(int numOfNodes, int coresPerProc,
      int procsPerNode, int nodesPerRack, int numRacks) :
      resourceallocator(numOfNodes, coresPerProc, procsPerNode, nodesPerRack,
          numRacks)
  {

    currentRack = 0;

  }

  rackallocator::~rackallocator()
  {

  }

  space*
  rackallocator::allocate(int num)
  {

    int endrack = min(
        (int) (currentRack + ceil(
            (double) num / (double) (system->coresPerProc * system->procsPerNode
                * system->nodesPerRack))), system->numRacks - 1);

    space* ret = new rackspace(currentRack, endrack, system->numNodes,
        system->coresPerProc, system->procsPerNode, system->nodesPerRack,
        system->numRacks);

    currentRack = (endrack + 1) % system->numRacks;

    return ret;
  }

} }
