/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include "systemspace.h"

namespace sstmac { namespace omnetpp {


  systemspace::systemspace(int nodes, int corepp, int ppn, int npr, int nr) :
    space(nodes, corepp, ppn, npr, nr)
  {


  }

  systemspace::~systemspace()
  {

  }

  omnetpp_address::ptr
  systemspace::begin_iteration()
  {
    current->rack(0);
    current->node(0);
    current->proc(0);
    current->core(0);

    return omnetpp_address::construct(current);
  }

  omnetpp_address::ptr
  systemspace::next()
  {
    current->core(current->core() + 1);

    if (current->core() >= coresPerProc)
      {

        current->proc(current->proc() + 1);
        current->core(0);

        if (current->proc() >= procsPerNode)
          {
            current->node(current->node() + 1);
            current->proc(0);

            if (numRacks >= 0 && current->node() >= nodesPerRack)
              {
                current->rack(current->rack() + 1);
                current->node(0);

                if (current->rack() > numRacks)
                  {
                    begin_iteration(); //reset it
                  }
              }
          }

      }

    return omnetpp_address::construct(current);
  }

  /*
 Address systemspace::getRankMap(int r) {
 int rackid = numRacks > 0 ? r
 / (coresPerProc * procsPerNode * nodesPerRack) : -1;
 Address ret(r % coresPerProc, (r / coresPerProc) % procsPerNode,
 (r / (coresPerProc * procsPerNode)) % nodesPerRack, rackid);
 //std::cout << "converting rank " << r << " to " << ret << std::endl;
 return ret;

 }

   */
} }
