/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/common/messages/eventdata.h>
#include <sstmac/common/errors.h>

namespace sstmac {


  eventdata::eventdata(std::string libn, timestamp t) : sst_message(libn, t, sst_message::SST, 1)
  {};



  eventdata::ptr 
  eventdata::construct(std::string libn, timestamp t)
  {
    eventdata::ptr 
      retval(new eventdata(libn, t));

   return retval;
  }



  unsigned long long int
  eventdata::get_event(std::string key)
  {
    if( data.find(key) == data.end() ){
        return 0;
    }
    return data[key]; 
  }


  bool
  eventdata::has_event(std::string key)
  {
    if( data.find(key) == data.end() )
      return false;
    return true;
  }

} // end of namespace sstmac
