/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/messages/sst_serializer.h>
#include <sstmac/common/richpair.h>

#include <sstmac/software/libraries/mpi/mpi_strategies/mpi_collective/mpicollpayload.h>
#include <sstmac/common/messages/valuepayload.h>

namespace sstmac
{

  payload::const_ptr
  sst_serializer::deserialize_content(sst_serializer* ser)
  {
    std::string name;
    ser->unpack(name); //unfortunately, this is the only supported method of deserialization right now

    if (name == payload::get_deser_name())
      {
        return payload::deserialize(ser);
      }
    else if (name == sstmac::sw::mpicollpayload::get_deser_name())
      {
        return sstmac::sw::mpicollpayload::deserialize(ser);
      }
    else if (name == valuepayload<int>::get_deser_name())
      { /// TODO: support more than just in payloads
        return valuepayload<int>::deserialize(ser);
      }
    else if (name == valuepayload<richpair<int, int> >::get_deser_name())
      { /// TODO: support more than just in payloads
        return valuepayload<richpair<int, int> >::deserialize(ser);
      }
    else
      {
        std::cout << "desername: " << name << std::endl;
        throw valueerror(
            "sst_serializer: unknown payload desername during deserialization");
      }

  }

} // end of namespace sstmac
