/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/stats/power_config.h>
#include <sstmac/common/errors.h>

#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <sstream>

namespace sstmac {

using namespace std;

power_config::power_config()
{
  proc_flop = 10E-15;
  proc_memop = 1E-13;
  proc_memop_local = 1E-12;
  proc_memop_rand = 3E-12;
  proc_static = 5;
  mem_static = 8;
  mem_access = 3E-12;

  rtr_static = 8;
  rtr_hop_per_byte = 16E-12;

}

/// Protected constructor using a file.
power_config::power_config(const std::string &filename)
{
  ifstream infile(filename.c_str());

  if (!infile.is_open())
    {
      stringstream error;
      error << "problem opening power config file: " << filename;
      throw valueerror(error.str());
    }

  string key, val;
  while (infile >> key)
    {
      infile >> val;
      if (key == "proc_flop")
        proc_flop = atof(val.c_str());
      else if (key == "proc_memop")
        proc_memop = atof(val.c_str());
      else if (key == "proc_memop_local")
        proc_memop_local = atof(val.c_str());
      else if (key == "proc_memop_rand")
        proc_memop_rand = atof(val.c_str());
      else if (key == "proc_static")
        proc_memop_rand = atof(val.c_str());

      else if (key == "rtr_static")
        rtr_static = atof(val.c_str());
      else if (key == "rtr_hop_per_byte")
        rtr_hop_per_byte = atof(val.c_str());

      else
        {
          stringstream error;
          error << "unrecognized option in power config file: " << key;
          throw valueerror(error.str());
        }
    }
}

/// Public constructor using a file.
power_config::ptr
power_config::construct(const std::string &filename)
{
  ptr retval(new power_config(filename));
  return retval;
}

power_config::ptr
power_config::construct()
{
  ptr retval(new power_config());
  return retval;

}

void
power_config::printPowerConfig(ostream &out)
{
  out << "--- Processor --- " << std::endl;
  out << "proc_static: " << proc_static << std::endl;
  out << "proc_memop: " << proc_memop << std::endl;
  out << "proc_memop_local: " << proc_memop_local << std::endl;
  out << "proc_memop_rand: " << proc_memop_rand << std::endl;

  out << "--- Memory --- " << std::endl;
  out << "mem_static: " << mem_static << std::endl;
  out << "mem_access: " << mem_access << std::endl;

  out << "--- Network --- " << std::endl;
  out << "rtr_static: " << rtr_static << std::endl;
  out << "rtr_hop_per_byte: " << rtr_hop_per_byte << std::endl;
}
} // end of namespace sstmac
