/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/network/congestion/interconnect.h>

namespace sstmac
{
  namespace hw
  {
    debug interconnect::dbg_;

    bool interconnect::registered_stats_ = false;
    stat_sum::ptr interconnect::pwrstat_;
    stat_time_window_sum_duration::ptr interconnect::energystat_;
    stat_time_window_sum_duration::ptr interconnect::bandwidthstat_;

    interconnect::interconnect(topology::ptr t) : topology_(t)
    {
      if(!dbg_.is_real()){
          dbg_ = debug("<debug> interconnect | network");
          logger::register_logger(&dbg_);
      }

      if(!pwrstat_){
          const std::string fname = "stats_interconnect_pwr.csv";
          pwrstat_ =  stat_sum::construct("<stats> interconnect | network", fname);
          register_stat(pwrstat_);
      }
      if(!energystat_){
          const std::string fname = "stats_interconnect_energy.csv";
          energystat_ =  stat_time_window_sum_duration::construct("<stats> interconnect | network", fname);
          register_stat(energystat_);
      }
      if(!bandwidthstat_){
          const std::string fname = "stats_interconnect_bandwidth.csv";
          bandwidthstat_ =  stat_time_window_sum_duration::construct("<stats> interconnect | network", fname);
          register_stat(bandwidthstat_);
      }
    }



  }
}
