/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_CONGESTION_SHAREDBUFFER_ARRIVAL_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_CONGESTION_SHAREDBUFFER_ARRIVAL_H_INCLUDED

#include <sstmac/hardware/network/congestion/sharedbuffer.h>
#include <sstmac/common/eventhandler.h>
#include <sstmac/hardware/network/topology/topology.h>

namespace sstmac { namespace hw  {

/**
 * Handles the arrival of the head of a flow to a particular vertex.
 */
class sharedbuffer::arrival: public eventhandler {
        /// Dear old papa sharedbuffer
        sharedbuffer::ptr parent_;

        /// The arriving flow.
        flow_id_t fid_;

        /// The vertex the flow is arriving at.
        vertex_index v_;

        /// Bandwidth flow is arriving with.
        double input_bw_;

        /// Flow size. Only really important at injection.
        double bytes_;

protected:
        /// Construct a new arrival event.
        arrival(sharedbuffer::ptr parent, flow_id_t fid,
                        vertex_index v, double input_bw, double bytes);

public:
        typedef boost::intrusive_ptr<arrival> ptr;

        /// Construct a new arrival event.
        static arrival::ptr
        construct(sharedbuffer::ptr parent, flow_id_t fid,
                        vertex_index v, double input_bw, double bytes);

        /// Destructor.
        virtual ~arrival() throw ();

        /// Handle the arrival of flow fid_ at vertex v_.
        virtual void handle(const sst_message::ptr &msg);

        virtual std::string to_string() const {
                return "sharedbuffer::arrival";
        }

};

} } // end of namespace sstmac

#endif
