/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_CONGESTION_SHAREDBUFFER_BANDWIDTHCHANGE_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_CONGESTION_SHAREDBUFFER_BANDWIDTHCHANGE_H_INCLUDED

#include <sstmac/hardware/network/congestion/sharedbuffer.h>
#include <sstmac/common/eventhandler.h>
#include <sstmac/hardware/network/topology/topology.h>

namespace sstmac { namespace hw  {

/**
 * Handles the state changes that could affect the bandwidth allocations on a particular edge.
 */
class sharedbuffer::bandwidthchange: public eventhandler {
        /// Our sharedbuffer.
        sharedbuffer::ptr parent_;
        /// Vertex event is occurring on.
        vertex_index v_;
        /// Output edge bandwidth is changing on.
        edge_index e_;

protected:
        /// Construct a new bandwidth change event.
        bandwidthchange(sharedbuffer::ptr parent, vertex_index v,
                        edge_index e);

public:
        typedef boost::intrusive_ptr<bandwidthchange> ptr;

        /// Construct a new bandwidth change event.
        static bandwidthchange::ptr
        construct(sharedbuffer::ptr parent, vertex_index v,
                        edge_index);

        /// Destructor.
        virtual ~bandwidthchange() throw ();

        /// Handle bandwidth changing on edge e_ at vertex v_.
        virtual void handle(const sst_message::ptr &msg);

        virtual std::string to_string() const {
                return "sharedbuffer::bandwidthchange";
        }

};

} } // end of namespace sstmac

#endif
