/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_CONGESTION_SHAREDBUFFER_BUFFERCHANGE_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_CONGESTION_SHAREDBUFFER_BUFFERCHANGE_H_INCLUDED

#include <sstmac/hardware/network/congestion/sharedbuffer.h>
#include <sstmac/common/eventhandler.h>
#include <sstmac/hardware/network/topology/topology.h>

namespace sstmac { namespace hw  {

/**
 * Handles the potential occurence of a flow either filling or draining its buffer.
 */
class sharedbuffer::bufferchange: public eventhandler {
        /// Our sharedbuffer
        sharedbuffer::ptr parent_;
        /// Vertex event is occurring on
        vertex_index v_;
        /// Flow having the buffer event
        flow_id_t fid_;

protected:
        /// Construct a new buffer event.
        bufferchange(sharedbuffer::ptr parent, vertex_index v,
                        flow_id_t fid);

public:
        typedef boost::intrusive_ptr<bufferchange> ptr;

        /// Construct a new buffer event.
        static bufferchange::ptr
        construct(sharedbuffer::ptr parent, vertex_index v,
                        flow_id_t fid);

        /// Destructor.
        virtual ~bufferchange() throw ();

        /// Handle the buffer event for flow fid_ at vertex v_.
        virtual void handle(const sst_message::ptr &msg);

        virtual std::string to_string() const {
                return "sharedbuffer::bufferchange";
        }

};

} } // end of namespace sstmac

#endif
