/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_PROCESSOR_COMPUTESCHEDULER_H_INCLUDED
#define SSTMAC_HARDWARE_PROCESSOR_COMPUTESCHEDULER_H_INCLUDED

#include <sstmac/common/eventhandler.h>
#include <sstmac/common/messages/sst_message.h>
#include <sstmac/common/eventmodule.h>
#include <sstmac/common/ptr_type.h>


#include <map>
#include <queue>

namespace sstmac { namespace hw 
  {

  class computescheduler : public eventmodule
  {
  protected:
    computescheduler(int ncores);

    int n_cores_;

    static debug dbg_;

    struct compute_job
    {
      compute_job() {}
      compute_job(sst_message::ptr m, timestamp t,
          eventhandler::ptr h, int c = -1) :
            msg_(m), time_(t), handle_(h), core_(c)
      {
      }

      sst_message::ptr msg_;
      timestamp time_;
      eventhandler::ptr handle_;
      int core_;
    };



  public:
    virtual std::string to_string() const {
      return "computescheduler";
    }
    typedef boost::intrusive_ptr<computescheduler> ptr;

    virtual
    ~computescheduler()
    {
    }

    static computescheduler::ptr
    construct(int ncores);

    void
    schedule_computation(const timestamp& comp,
        const sst_message::ptr& to_deliver,
        eventhandler::ptr &handler);

    virtual void
    handle(const sst_message::ptr &msg);

  private:
    std::map<sst_message::ptr, compute_job> slots_compute_;
    std::map<int, bool> compute_slots_;
    std::queue<compute_job> suspendedQ;


  };

  }  } //end of namespace sstmac
#endif
