/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/compute/lib_compute_inst.h>

#include <sstmac/software/process/key.h>


namespace sstmac { namespace sw
{

  debug lib_compute_inst::dbg_;


  std::string
  lib_compute_inst::get_lib_name()
  {
    std::stringstream ss;
    ss << "computelibinstr" << id_.to_string();
    return ss.str();
  }

  lib_compute_inst::lib_compute_inst(softwareid id) :
      library(id)
  {
    if(!dbg_.is_real()){
         dbg_ = debug("<debug> lib_compute_inst | lib_compute");
         logger::register_logger(&dbg_);
     }
  }

  lib_compute_inst::~lib_compute_inst()
  {

  }

  lib_compute_inst::ptr
      lib_compute_inst::construct(softwareid id){
    return ptr(new lib_compute_inst(id));
  }

  void
  lib_compute_inst::handle(const sst_message::ptr &msg)
  {
    if (keys_.find(msg) == keys_.end())
      {
        throw libraryerror(
            "lib_compute_inst: message arrived which is not in our key list");
      }

    SSTMAC_DEBUG << "lib_compute_inst: received message, unblocking \n";

    key::ptr key = keys_[msg];
    os_->unblock(key);
  }

  void
  lib_compute_inst::compute_inst(
      const eventdata::ptr &cmsg)
  {
    if (supported())
      {

        if(cmsg->get_lib_name() == ""){
            cmsg->set_lib_name(get_lib_name());
        }
        key::ptr key = key::construct();
        keys_[cmsg] = key;
        SSTMAC_DEBUG << "lib_compute_inst: executing compute kernel \n";

        os_->execute_kernel(AMI::AMI_COMPUTE, AMI::AMI_COMP_INSTR, cmsg);

        SSTMAC_DEBUG << "lib_compute_inst: blocking \n";
        os_->block(key);

      }
    else
      {
        throw valueerror(
            "lib_compute_inst requires the presence of a processor model which can compute instructions");
      }
  }

} } //end of namespace sstmac
