/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_LIB_COMPUTE_INST_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_LIB_COMPUTE_INST_H_INCLUDED

#include <sstmac/software/libraries/library.h>
#include <sstmac/software/process/softwareid.h>
#include <sstmac/common/sstmac-config.h>
#include <sstmac/common/messages/eventdata.h>

#include <map>

//these are the default instruction labels
#define INST_FLOP_LABEL "flop"
#define INST_MEMOP_LABEL "memop"
#define INST_MEMOP_LOCAL_LABEL "memop_local"
#define INST_MEMOP_RAND_LABEL "memop_rand"

namespace sstmac { namespace sw
{

  class lib_compute_inst : public library
  {
  public:
    typedef boost::intrusive_ptr<lib_compute_inst> ptr;
    // ------- constructor / boost stuff -------------//

  protected:
    lib_compute_inst(softwareid id);



  public:


    virtual
    ~lib_compute_inst();

    // --------------------------------------//

  public:

    void
    compute_inst(const eventdata::ptr &msg);

    static lib_compute_inst::ptr
    construct(softwareid id);

    virtual void
    handle(const sst_message::ptr &msg);

    virtual std::string
    get_lib_name();

    virtual bool
    supported()
    {
      return os_->kernel_supported(AMI::AMI_COMPUTE, AMI::AMI_COMP_INSTR);
    }

  protected:

    std::map<sst_message::ptr, key::ptr > keys_;

    static debug dbg_;
  };

} } //end of namespace sstmac

#endif
