/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <math.h>

#include <sstmac/software/libraries/compute/lib_compute_matrix.h>

#include <sstmac/software/process/key.h>

namespace sstmac { namespace sw
{

  std::string
  lib_compute_matrix::get_lib_name()
  {
    std::stringstream ss;
    ss << "computelibmatrix" << id_.to_string();
    return ss.str();
  }

  lib_compute_matrix::lib_compute_matrix(softwareid id) :
      lib_compute_inst(id)
  {

  }

  //
  // Build a new minimd object.
  //
  lib_compute_matrix::ptr
  lib_compute_matrix::construct(softwareid id)
  {
    lib_compute_matrix::ptr retval(new lib_compute_matrix(id));
    // retval->set_self(retval);
    return retval;
  }

  lib_compute_matrix::~lib_compute_matrix()
  {

  }
  
  void
  lib_compute_matrix::double_mxm(long nrow, long nlink, long ncol,
                                 long nthread)
  {
    double_compute(nrow*nlink*ncol, nrow*ncol+nrow*nlink+ncol*nlink,
                   nrow*ncol, nrow*ncol,
                   0, 0, nrow*nlink*ncol,
                   nthread);
  }

  void
  lib_compute_matrix::double_add(long nrow, long ncol, long nthread)
  {
    double_compute(2*nrow*ncol, 2*nrow*ncol,
                   nrow*ncol, nrow*ncol,
                   nrow*ncol, 0, 0, nthread);
  }

  void
  lib_compute_matrix::double_copy(long nrow, long ncol, long nthread)
  {
    double_compute(nrow*ncol, nrow*ncol,
                   nrow*ncol, nrow*ncol,
                   0, 0, 0, nthread);
  }

  void
  lib_compute_matrix::double_compute(long ndata_read, long working_set_size_read,
                                   long ndata_write, long working_set_size_write,
                                   long nadd, long nmult,
                                   long nmultadd, long nthread)
  {
    eventdata::ptr inst = eventdata::construct(get_lib_name(),
                                               timestamp(0));
    inst->set_event(INST_MEMOP_LABEL,
                    ceil(double(working_set_size_read + working_set_size_write)/nthread));
    inst->set_event(INST_FLOP_LABEL,
                    ceil(double(nadd + nmult + 2*nmultadd)/nthread));
    compute_inst(inst);
  }
                                   

} } //end of namespace sstmac
