/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/compute/lib_compute_memmove.h>

#include <sstmac/software/process/key.h>


namespace sstmac { namespace sw
{

  using namespace std;

  std::string
  lib_compute_memmove::get_lib_name()
  {
    std::stringstream ss;
    ss << "computelibmemmove" << id_.to_string();
    return ss.str();
  }

  lib_compute_memmove::lib_compute_memmove(softwareid id) :
      lib_compute_inst(id)
  {

  }

  //
  // Build a new minimd object.
  //
  lib_compute_memmove::ptr
  lib_compute_memmove::construct(softwareid id)
  {
    lib_compute_memmove::ptr retval(new lib_compute_memmove(id));
    // retval->set_self(retval);
    return retval;
  }

  lib_compute_memmove::~lib_compute_memmove()
  {

  }

  void
  lib_compute_memmove::compute_block_access(int bytes)
  {
    eventdata::ptr inst = eventdata::construct(get_lib_name(),
        timestamp(0));


    inst->set_event(INST_MEMOP_LABEL, max(1,bytes / BLOCK_SIZE));

    compute_inst(inst);
  }



} } //end of namespace sstmac
