/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_LIB_COMPUTE_TIME_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_LIB_COMPUTE_TIME_H_INCLUDED

#include <sstmac/software/libraries/library.h>
#include <sstmac/software/process/softwareid.h>
#include <sstmac/common/sstmac-config.h>

#include <map>

namespace sstmac { namespace sw
{

  class lib_compute_time : public library
  {
    // ------- constructor / boost stuff -------------//


  protected:
    lib_compute_time(softwareid id);

    /// Set the "this is you" pointer.
    // void
    // set_self(const lib_compute_time::ptr&);

    /// Get the "this is me" pointer.
    // lib_compute_time::ptr
    // get_self();

  public:
    typedef boost::intrusive_ptr<lib_compute_time> ptr;

    virtual
    ~lib_compute_time();

    static lib_compute_time::ptr
    construct(softwareid id);
    // --------------------------------------//

  public:

    void
    compute(timestamp time);

    virtual void
    handle(const sst_message::ptr &msg);

    virtual std::string
    get_lib_name();

    virtual bool
    supported()
    {
      return os_->kernel_supported(AMI::AMI_COMPUTE, AMI::AMI_COMP_TIME);
    }

  protected:

    std::map<sst_message::ptr, key::ptr > keys_;

    static debug dbg_;
  };

} } //end of namespace sstmac

#endif
