/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_LIBRARY_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_LIBRARY_H_INCLUDED

#include <sstmac/software/process/softwareid.h>
#include <sstmac/software/process/os_api.h>
#include <sstmac/common/debug.h>
#include <sstmac/common/messages/sst_message.h>
#include <sstmac/common/eventhandler.h>
#include <sstmac/common/ptr_type.h>

namespace sstmac {
namespace sw {

class library: public eventhandler {

protected:
	library(softwareid id) :
			id_(id) {

	}

public:
	typedef boost::intrusive_ptr<library> ptr;

	virtual std::string to_string() const {
		return "library";
	}

	virtual std::string
	get_lib_name() = 0;

	void init_os(OS_API::ptr os) {
		os_ = os;
	}

	//convenience function
	virtual void block(key::ptr &k) {
		os_->block(k);
	}

	//convenience function
	virtual void unblock(key::ptr &k) {
		os_->unblock(k);
	}

	virtual bool supported() {
		return true;
	}
protected:

	OS_API::ptr os_;

	softwareid id_;

};

}
} //end of namespace sstmac

#endif
