/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_MPICOLLPAYLOAD_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_MPICOLLPAYLOAD_H_INCLUDED

#include <sstmac/common/messages/payload.h>

#include <vector>

namespace sstmac { namespace sw
{

  class mpicollpayload : public payload
  {

  public:
    typedef boost::intrusive_ptr<mpicollpayload> ptr;
    typedef boost::intrusive_ptr<const mpicollpayload> const_ptr;

    static mpicollpayload::ptr
    construct(const std::vector<payload::const_ptr > & l)
    {
      mpicollpayload::ptr ret(new mpicollpayload(l));
      return ret;
    }

    static mpicollpayload::ptr
    construct(const const_ptr &p)
    {
      mpicollpayload::ptr ret(new mpicollpayload(p->get_content()));
      return ret;
    }

    virtual
    ~mpicollpayload() throw ()
    {
    }

    std::vector<payload::const_ptr >
    get_content() const
    {
      return contents_;
    }

    void
    set_content(const payload::const_ptr &p, int index)
    {
      contents_[index] = p;
    }

    virtual std::string to_string() const{
            return "mpicollpayload";
          }

    virtual payload::const_ptr
    clone() const
    {

      payload::const_ptr ret(new mpicollpayload(contents_));
      return ret;
    }

    virtual const payload::const_ptr
    add(const payload::const_ptr &other) const
    {

      throw unimplementederror(
          "mpicollpayload:add doesn't make a whole lot of sense.");
    }

    virtual bool
    less_than(const payload::const_ptr &other) const {
      throw unimplementederror(
          "mpicollpayload:operator< doesn't make a whole lot of sense.");

    }

    virtual bool
    greater_than(const payload::const_ptr &other) const {
      throw unimplementederror(
          "mpicollpayload:operator> doesn't make a whole lot of sense.");

    }

    virtual bool
    equals(const payload::const_ptr &other) const {
      throw unimplementederror(
          "mpicollpayload:operator== doesn't make a whole lot of sense.");

    }

    virtual void
    serialize(sst_serializer* ser) const
    {
      ser->pack(get_deser_name());
      ser->pack(contents_.size());
      std::vector<payload::const_ptr >::const_iterator it, end =
          contents_.end();

      SSTMAC_DEBUG << "mpicollpayload: serializing " << contents_.size() << " payloads \n";

      for (it = contents_.begin(); it != contents_.end(); it++)
        {
          bool hasContent = false;
          if (*it)
            {
              hasContent = true;
              ser->pack(hasContent);
               SSTMAC_DEBUG << "serializing content: " << (*it)->to_string() << "\n";
              (*it)->serialize(ser);
            }
          else
            {
              ser->pack(hasContent);
            }
        }
    }

    static payload::const_ptr
    deserialize(sst_serializer* ser)
    {
      std::vector<payload::const_ptr > vec;
      size_t num;
      ser->unpack(num);

      SSTMAC_DEBUG << "mpicollpayload: deserializing with " << num << " internal payloads \n";

      for (size_t i = 0; i < num; i++)
        {
          bool has;
          ser->unpack(has);
          if (has)
            {

              SSTMAC_DEBUG << "mpicollpayload: deserializing a real payload \n";
              payload::const_ptr pay = sst_serializer::deserialize_content(ser);
              SSTMAC_DEBUG << "mpicollpayload: got " << pay->to_string() << "\n";
              vec.push_back(pay);
            }
          else
            {
              vec.push_back(payload::null());
            }
        }

      return construct(vec);
    }

    static std::string
    get_deser_name()
    {
      return "mpicollpayload";
    }

  protected:
    std::vector<payload::const_ptr > contents_;

    mpicollpayload(const std::vector<payload::const_ptr > & l) :
      contents_(l)
    {

    }
  };

} }

#endif
