/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/mpi/mpi_strategies/mpicorealltoall.h>

#include <sstmac/software/libraries/unblockhandler.h>
#include <sstmac/software/libraries/mpi/mpi_strategies/mpi_collective/engines/mpidirectalltoall.h>

namespace sstmac { namespace sw
{

  // Pointer types.
  

  //
  // Hi.
  //
  mpicorealltoall::mpicorealltoall()
  {
  }

  //
  // Hello.
  //
  mpicorealltoall::ptr
  mpicorealltoall::construct()
  {
    return mpicorealltoall::ptr (new mpicorealltoall);
  }

  //
  // Must be virtual.
  //
  mpicorealltoall::~mpicorealltoall() throw ()
  {
  }

  //
  // Build a kernel.
  //
  void
  mpicorealltoall::execute(const mpirequest_t& thekey, const
      mpiqueue::ptr& queue, int sendcnt, const mpitype &sendtype, int recvcnt,
      const mpitype &recvtype, const mpitag &tag, const mpicomm &comm,
      const std::vector<payload::const_ptr >& content,
      OS_API::ptr &os) const
  {
     std::vector<int> sendvec(comm.size().id, sendcnt);
     std::vector<int> recvvec(comm.size().id, recvcnt);
    // return mpidirectalltoall::construct
    //   (thekey, queue, sendvec, sendtype, recvvec, recvtype,
    //    tag, comm, content, magic);

    key::ptr k = thekey->get_key();
    eventhandler::ptr completion = unblockhandler::construct(os, k);
    mpidirectalltoall::ptr engine = mpidirectalltoall::construct(
        thekey, queue, sendvec, sendtype, recvvec, recvtype, tag, comm,
        content, completion);

    engine->start();
  }

} } // end of namespace sstmac
