/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/mpi/mpi_strategies/mpicorescatter.h>

#include <sstmac/software/libraries/unblockhandler.h>
#include <sstmac/software/libraries/mpi/mpi_strategies/mpi_collective/engines/mpibtreescatter.h>

#include <sstmac/common/errors.h>
#include <sstmac/common/debug.h>

namespace sstmac { namespace sw
{

  

  //
  // Hi.
  //
  mpicorescatter::mpicorescatter()
  {
  }

  //
  // Hello.
  //
  mpicorescatter::ptr
  mpicorescatter::construct()
  {
    return mpicorescatter::ptr (new mpicorescatter);
  }

  //
  // Perform a fused reduce-scatter operation
  //
  void
  mpicorescatter::execute(const mpirequest_t& thekey,
      const mpiqueue::ptr& queue, int sendcount,
      const mpitype &sendtype, int recvcount, const mpitype &recvtype,
      const mpiid &root, const mpitag &tag, const mpicomm &comm,
      const std::vector<payload::const_ptr > &cc, OS_API::ptr &os) const
  {
    //return mpibtreescatter::construct(thekey, queue, sendcount, sendtype,
    //                                 recvcount, recvtype, root, tag, comm, cc);

    key::ptr k = thekey->get_key();
    eventhandler::ptr completion = unblockhandler::construct(os, k);
    mpibtreescatter::ptr engine = mpibtreescatter::construct(
        thekey, queue, sendcount, sendtype, recvcount, recvtype, root, tag,
        comm, cc, completion);

    engine->start();
  }

} } // end of namespace sstmac
