/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_PROCESS_APP_H_INCLUDED
#define SSTMAC_SOFTWARE_PROCESS_APP_H_INCLUDED

#include <sstmac/software/libraries/compute/lib_compute_inst.h>
#include <sstmac/software/libraries/compute/lib_compute_time.h>
#include <sstmac/software/libraries/compute/lib_sleep.h>
#include <sstmac/common/ptr_type.h>
#include <sstmac/software/process/thread.h>
//#include <sstmac/hardware/processor/eventdata.h>
#include <list>

namespace sstmac { namespace sw
{


  /**
   * The app derived class adds to the thread base class by providing
   * facilities to allow applications to simulate computation.  
   * Messaging models are supported through derivation of this class,
   * as in the mpiapp class.
   */
  class app : public thread
  {
    lib_compute_inst::ptr computeinst_;
    lib_compute_time::ptr computetime_;
    lib_sleep::ptr sleeplib_;

  protected:
    app(softwareid id)  : id_(id)
    {

    }

    softwareid id_;

    //convenience function for apps
    void
    compute(timestamp time);

    void compute_inst(
        const eventdata::ptr &cmsg);

    void
    sleep(timestamp time);

  public:
    typedef boost::intrusive_ptr<app> ptr;

    /// Goodbye.
    virtual
    ~app()
    {
    }

    virtual void
    consume_params(const std::list<std::string> &params) = 0;

    virtual app::ptr
    clone(softwareid newid) = 0;

    //called when killing the app, in case you want to check or clean anything up before destructor
    virtual void
    kill()
    {

      thread::kill();
    }

    virtual void
    skeleton_main() = 0;

    virtual void
    run(){
      skeleton_main();
    }
  };

} } // end of namespace sstmac.

#endif
