/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_PROCESS_KEY_H_INCLUDED
#define SSTMAC_SOFTWARE_PROCESS_KEY_H_INCLUDED

//#include <sstmac/software/threading/context.h>
#include <sstmac/software/threading/threading_interface.h>

#include <sstmac/common/ptr_type.h>
#include <vector>

namespace sstmac { namespace sw {

  /**
   * A base type and default (empty) implementation of a handle
   * to block and unblock processes.
   */
  class key : public ptr_type {
    friend class operatingsystem;
    typedef std::vector< threading_interface* > blocking_t;
    blocking_t blocking_;
  public:

    virtual std::string to_string() const{
            return "key";
          }


    typedef boost::intrusive_ptr<key> ptr;

    static key::ptr construct();
    virtual ~key() {
      //std::cout << "calling key destructor" << std::endl;
    }

    void
    add_blocker(threading_interface* t){
      blocking_.push_back(t);
    }

    bool
    still_blocked(){
      return blocking_.size() > 0;
    }

    void clear(){
      blocking_.clear();
    }
  };

} } // end of namespace sstmac

#endif
