/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/process/thread.h>
#include <sstmac/software/process/os_api.h>
#include <sstmac/software/process/key.h>
#include <sstmac/common/errors.h>
#include <iostream>
#include <exception>
#include <unistd.h>  // getpagesize()
#include <sys/mman.h>
#include <memory.h>
#include <stdlib.h>
#include <stdio.h>
//#include <signal.h>
//#include <dlfcn.h> // dladdr (not a POSIX call)
//#include <cxxabi.h> // Demangle C++ names (not a POSIX call)

namespace sstmac { namespace sw {

  
  
  debug thread::dbg_;

  //
  // Private method that gets called by the scheduler.
  //
  void thread::init_thread(threading_interface* threadcopy, void *stack, int stacksize, OS_API::ptr os,
                    threading_interface *yield_to)
  {
    stack_ = stack;
    stacksize_ = stacksize;
    os_ = os;

    isInit = true;

    //there were libs that wanted to be registered
    if(!pending_libs_.empty()){
        std::list<library::ptr >::iterator it;
        for(it = pending_libs_.begin(); it != pending_libs_.end(); it++){
            os_->register_lib(*it);
        }
    }

    pending_libs_.clear();

    context_ = threadcopy->copy();

    context_->start_context(stack, stacksize, run_routine, this, yield_to);


  }
  
  void thread::destroy()
  {
    context_->destroy_context();

    delete context_;
  }
  
  //
  // Run routine that defines the initial context for this task.
  //
  void thread::run_routine(void *threadptr) {
    thread *task = (thread*)threadptr;

    assert(task != 0);
    /// This locks the reference counted pointer, ensuring that this
    /// object will not be erased before its time.
    //thread::ptr self = task->get_self();
    thread* self = task;
    if(self == 0) {
      // You silly person -- you probably forgot to call thread::set_self.
      throw nullerror("thread::init:  thread::get_self returns a null pointer."
                      "  You need to call thread::set_self when you construct "
                      "a new (derived) thread object.");
    }
    // Go.
    if(self->initialized()) {
      self->state_ = ACTIVE;
      self->run();
      // We are done, ask the scheduler to remove this task from the
      // list of active tasks.
      self->state_ = DONE;
     // self->get_scheduler()->unblock(self->key_);
     // self->get_scheduler()->complete(self->stack_);
      self->os_->complete_thread(self);
    }
    else {
      throw oserror("thread::run_routine:  Task has not "
                           "been initialized.");
    }
  }

  //
  // Constuctor.
  //
  thread::thread() :
    state_(PENDING), isInit(false),  key_(key::construct())
  {
    if(!dbg_.is_real()){
         dbg_ = debug("<debug> thread | app", logger::green);
         logger::register_logger(&dbg_);
     }
  }



  //
  // Goodbye.
  //
  thread::~thread() {
  }

  //
  // Start a new thread.
  //
  void thread::start_thread(const thread::ptr& thr) {
    os_->start_thread(thr);
  }

  //
  // Join threads.
  //
  void thread::join() {
    if(! this->initialized()) {
      // We can't context switch the caller out without a scheduler.
      throw illformederror("thread::join():  Target thread "
                           "has not been initialized.");
    }
   // os_->join_thread(get_self(), context_);
  }



} } // end of namespace sstmac
