/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/services/launch/indexing/roundrobinindexing.h>
#include <sstmac/common/errors.h>

namespace sstmac  { namespace sw
  {



  roundrobinindexing::roundrobinindexing() {
  }

  roundrobinindexing::ptr roundrobinindexing::construct() {
    return roundrobinindexing::ptr(new roundrobinindexing);
  }

  roundrobinindexing::~roundrobinindexing() throw() {
  }

  void roundrobinindexing::allocate(const std::set<nodeaddress::ptr, nodeaddress::nodeaddresscomp> &nodes,
                                    int ppn,
                                    std::vector<nodeaddress::ptr> &result,
                                    int nproc)
  {
    if(nodes.empty()) {
      throw valueerror("roundrobinindexing::allocate:  Empty node vector.");
    }
    if(ppn <= 0) {
      throw valueerror("roundrobinindexing::allocate:  ppn must be > 0.");
    }
    if(ppn*nodes.size() != nproc) {
      throw valueerror("roundrobingindexing::allocate: currently require ppn*nodes.size() == nproc");
    }


    for(int i = 0; i < ppn; ++i) {
      result.insert(result.end(), nodes.begin(), nodes.end());
    }
  }

} } // end of namespace sstmac
