/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_LAUNCH_launchinfo_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_LAUNCH_launchinfo_H_INCLUDED

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/timestamp.h>
#include <list>
#include <sstmac/software/process/appid.h>

#include <sstmac/software/skeletons/skeletonfactory.h>

namespace sstmac
{
  namespace sw
  {

    class launchinfo : public ptr_type
    {
    public:
      typedef boost::intrusive_ptr<const launchinfo> ptr;

      timestamp start_;

      app::ptr apptype_;

      appid aid_;

      long num_tasks_;

      const std::list<std::string> params_;

    protected:
      launchinfo(app::ptr app, appid a, const std::list<std::string> &p,
          timestamp start, long num) :
          apptype_(app), aid_(a), params_(p), start_(start), num_tasks_(num)
      {

      }



    public:
      static ptr
      construct(std::string t, appid a, const std::list<std::string> &p,
          timestamp start, long num)
      {
        skeletonfactory::init_skeletons();
        app::ptr theapp = skeletonfactory::apps[t];
        return ptr(new launchinfo(theapp, a, p, start, num));
      }

      static ptr
      construct(app::ptr app, appid a, const std::list<std::string> &p,
          timestamp start, long num)
      {
        return ptr(new launchinfo(app, a, p, start, num));
      }

      virtual std::string
      to_string() const {
        return "launchinfo";
      }

      virtual ~launchinfo() {}
    };

  }
}
#endif
