/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SERVICES_SERVICE_H_INCLUDED
#define SSTMAC_SOFTWARE_SERVICES_SERVICE_H_INCLUDED

#include <sstmac/software/process/softwareid.h>
#include <sstmac/software/process/operatingsystem.h>
#include <sstmac/common/debug.h>
#include <sstmac/common/messages/sst_message.h>
#include <sstmac/common/eventhandler.h>
#include <sstmac/software/process/thread.h>
#include <sstmac/common/ptr_type.h>

namespace sstmac
{
  namespace sw
  {

    class service : public eventhandler, public thread
    {

    protected:
      service()
      {

      }

    public:
      typedef boost::intrusive_ptr<service> ptr;

      virtual std::string
      to_string() const
      {
        return "service";
      }

      virtual std::string
      get_service_name() = 0;

      //convenience function
      virtual void
      block(key::ptr &k)
      {
        os_->block(k);
      }

      //convenience function
      virtual void
      unblock(key::ptr &k)
      {
        os_->unblock(k);
      }

      virtual service::ptr
      clone() = 0;

      virtual void
      init_os(const operatingsystem::ptr &o){
        os_ = o;
      }
    protected:

      operatingsystem::ptr os_;

      bool is_app_init(appid aid){
        return  (os_->env_.find(aid) == os_->env_.end());
      }

    };

  }
} //end of namespace sstmac

#endif
