/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_MINIMD_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_MINIMD_H_INCLUDED

#include <sstmac/software/libraries/mpi/mpiapp.h>
#include <sstmac/software/skeletons/mini/minimd/lib_compute_minimd.h>
#include <string>

namespace sstmac { namespace sw
{


  /**
   * Early pass at turning miniMD into a skeleton application.
   */
  class minimd : public mpiapp
  {
    /// Nested types mirroring the layout of minimd.
    class in;
    class atom;
    class force;
    class neighbor;
    class integrate;
    class thermo;
    class comm;
    class timer;

    /// The input file name.
    std::string infile_;
    /// The interpolator/lookup table for our execution times.
    lib_compute_minimd::ptr execution_;
    /// The nested private objects.
    boost::intrusive_ptr<in> in_;
    boost::intrusive_ptr<atom> atom_;
    boost::intrusive_ptr<force> force_;
    boost::intrusive_ptr<neighbor> neighbor_;
    boost::intrusive_ptr<integrate> integrate_;
    boost::intrusive_ptr<thermo> thermo_;
    boost::intrusive_ptr<comm> comm_;
    boost::intrusive_ptr<timer> timer_;

  private:
    /// Mirror static functions in mpiMD.
    void
    input();
    void
    create_box();
    void
    create_atoms();
    void
    create_velocity();
    void
    output();

  protected:
    /// Hi.
    explicit
        minimd(softwareid id);

  public:
    typedef boost::intrusive_ptr<minimd> ptr;

    /// Build a new minimd object, which will read its input from
    /// the given file.
    static minimd::ptr
    construct(softwareid id);

    /// Goodbye.
    virtual
    ~minimd() throw ();

    /// Get a copy of this object.
    app::ptr
    clone(softwareid newid);

    /// We use init_mpi to initialize a bunch of other stuff as well.
    void
    init_minimd();

    /// Go.
    void
    skeleton_main();

    virtual void consume_params(const std::list<std::string> &params);
  };

} } // end of namespace sstmac

#endif
