/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2011 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_MINI_SYSTOLICMXM_SYSTOLICMXM_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_MINI_SYSTOLICMXM_SYSTOLICMXM_H_INCLUDED

#include <sstmac/software/libraries/mpi/mpiapp.h>
#include <sstmac/software/libraries/mpi/mpiapi.h>
#include <sstmac/software/libraries/compute/lib_compute_matrix.h>
#include <vector>
#include <map>
#include <math.h>
#include <stdlib.h>

namespace sstmac { namespace sw {

  /**
   * Torus-wrap Matrix Multiply.
   */
  class systolicmxm : public mpiapp {
  public:
    long nrowblocks;
    long nlnkblocks;
    long ncolblocks;
    long blockrowsize;
    long blocklnksize;
    long blockcolsize;
    long nrownodes;
    long ncolnodes;
    long nrowcorespernode;
    long ncolcorespernode;
    bool systolicsetup;
    long threads;
    lib_compute_matrix::ptr compute_matrix_;

  private:

    systolicmxm(softwareid id,
                long nrowblocks,
                long nlnkblocks,
                long ncolblocks,
                long blockrowsize,
                long blocklnksize,
                long blockcolsize,
                long nrownodes,
                long ncolnodes,
                long nrowcorespernode,
                long ncolcorespernode,
                bool systolicsetup,
                long threadsperrank);

    /// Convert grid coordinates to a rank.
    /// Out of range blocks are wrapped a single time.
    void grid_to_rank(long row, long col, long &rank);
    /// Convert a rank to grid coordinates.
    /// Out of range blocks are wrapped a single time.
    void rank_to_grid(long rank, long &row, long &col);

  public:
    typedef boost::intrusive_ptr<systolicmxm> ptr;
    static ptr construct(softwareid id,
                         long nrowblocks,
                         long nlnkblocks,
                         long ncolblocks,
                         long blockrowsize,
                         long blocklnksize,
                         long blockcolsize,
                         long nrownodes,
                         long ncolnodes,
                         long nrowcorespernode,
                         long ncolcorespernode,
                         bool systolicsetup,
                         long threadsperrank);

    ~systolicmxm() throw();

    app::ptr clone(softwareid id);

    void skeleton_main();
    
    virtual void
    consume_params(const std::list<std::string> &params);

  };

}}

#endif

