/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/skeletons/test/mpipairwise.h>
#include <sstmac/software/libraries/mpi/mpiapi.h>
#include <string>
#include <stdio.h>
#include <sstream>
#include <iostream>
#include <fstream>

using std::string;

#define FILENAME "mpiconfig.txt"

namespace sstmac { namespace sw {

  // Pointer types.
  

  //
  // Hi.
  //
  mpipairwise::mpipairwise(softwareid id, int iterations, int count) :
    mpiapp(id), iterations_(iterations), count_(count)
  {}

  //
  // Hello. Reads in config from a file.
  // File format: <from id> <to id> <time since last event>
  //
  mpipairwise::ptr mpipairwise::construct(softwareid id, int iterations, int count) {
    mpipairwise::ptr obj(new mpipairwise(id, iterations, count));
    // obj->set_self(obj);

    std::string line;
    std::ifstream myfile;
    myfile.open(FILENAME);
    while(!myfile.eof()){
      std::getline(myfile, line);
      std::istringstream iss(line);
      std::string temp;
      iss >> temp;
      obj->from_.push_back(mpiid(atoi(temp.c_str())));
      iss >> temp;
      obj->to_.push_back(mpiid(atoi(temp.c_str())));
      iss >> temp;
      obj->when_.push_back(timestamp(atof(temp.c_str())));
    }
    myfile.close();

    return obj;
  }

  // Clone reproduction
  mpipairwise::ptr mpipairwise::clone_construct(softwareid id, int iterations, int count, mpipairwise* original) {
    mpipairwise::ptr obj(new mpipairwise(id, iterations, count));
    // obj->set_self(obj);
    obj->from_.resize(original->from_.size());
    obj->to_.resize(original->to_.size());
    obj->when_.resize(original->when_.size());
    for(int i = 0; i < obj->from_.size(); i++){
      obj->from_[i] = original->from_[i];
      obj->to_[i] = original->to_[i];
      obj->when_[i] = original->when_[i];
    }
    return obj;
  }

  //
  // Goodbye.
  //
  mpipairwise::~mpipairwise() throw() {
  }

  //
  // Get a copy.
  //
  sstmac::sw::app::ptr mpipairwise::clone(softwareid id) {
    return mpipairwise::clone_construct(id, iterations_, count_, this);
  }

  //
  // Go.
  //
  void mpipairwise::skeleton_main() {
    if(mpi() == 0) {
      throw nullerror("mpipairwise::run:  mpiapi pointer is null.");
    }
    mpicomm world = mpi()->comm_world();
    timestamp start = mpi()->init();
    mpiid rank = world.rank();
    mpiid size = world.size();
    //std::cerr << "mpipairwise[" << rank << "]::run(): mpi()->init() at " << start << "\n";
    mpitag tag(0);

    for(int i=0; i < from_.size(); i++){
      //fixme: need to add a computation delay here:
      //mpi()->compute(when_[i]);
      if(rank == from_[i]){
        // do send 
        mpi()->send(count_, mpitype::mpi_double, to_[i], tag, world);
      }
      if(rank == to_[i]){
        // do receive
        mpistatus::const_ptr stat = mpistatus::construct();
        mpi()->recv(count_, mpitype::mpi_double, from_[i], tag, world, stat);
      }
    }

    mpi()->finalize();

    //std::cerr << "mpipairwise[" << rank << "]::run():  mpi()->finalize() at " << done << "\n";
  }

  void
  mpipairwise::consume_params(const std::list<string> &params)
  {
  }

} } //end of namespace sstmac
