/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

//#include <sstmac/backends/native/launch/fixedindexing.h>
//#include <sstmac/backends/native/launch/allocationstrategy.h>
#include <bin/undumpi.h>
#include <sstmac/software/skeletons/undumpi/parsedumpi.h>
#include <sstmac/software/skeletons/undumpi/parsedumpi-callbacks.h>
#include <sstmac/software/skeletons/undumpi/dumpimeta.h>
#include <sstmac/common/errors.h>
#include <boost/lexical_cast.hpp>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <fstream>
#include <algorithm>

namespace sstmac { namespace sw {


  using namespace std;

  //
  // Hi there.
  //
  parsedumpi::parsedumpi(softwareid id) : mpiapp(id)

  {}
  
  //
  // Good news everyone!  We are getting a new parsedumpi.
  //
  parsedumpi::ptr
  parsedumpi::construct(softwareid id) {
    parsedumpi::ptr retval(new parsedumpi(id));

    return retval;
  }



  void parsedumpi::consume_params(const std::list<string> &params) {
    if (params.size() != 2) {
        std::cout << "parsedumpi: found " << params.size() << " parameters"
            << endl;
        throw valueerror("parsedumpi: expecting 2 parameters");
    }
    list<string>::const_iterator it = params.begin();

    fileroot_ = (*it);
    it++;
    timescaling_ = atof((*it).c_str());

  }

  //
  // Wait!  That's not good news at all!
  //
  parsedumpi::~parsedumpi() throw() {
  }

  //
  // IT'S AN ADDAMS!
  //
  app::ptr parsedumpi::clone(softwareid newid) {
    // put if to check fileroot
    return construct(newid);
  }




  //
  // Parse the file please (this needs an update, there is no reason
  // for it to return a TRACEREC (which should not be uppercase).
  // Need to figure out how to give the caller access to header/footer recs.
  //
  void parsedumpi::skeleton_main() {
    mpi()->init();

   // if((mpi() == 0) || (world().rank().id < 0))
   //   throw illformederror("parsedumpi::run:  Dumpi parser not initialized.");
    callbacks cbacks(this);
    std::string fname = dumpi_file_name(world().rank().id, fileroot_);
    // Ready to go.
    cbacks.parse_stream(fname.c_str());
  }

} } // end of namespace sstmac

