/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_UNDUMPI_PARSEDUMPI_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_UNDUMPI_PARSEDUMPI_H_INCLUDED

//#include <sstmac/backends/native/launch/allocationstrategy.h>
//#include <sstmac/backends/native/launch/indexstrategy.h>
//#include <sstmac/hardware/network/topology/topology.h>
#include <sstmac/software/libraries/mpi/mpiapp.h>
#include <sstmac/software/skeletons/undumpi/dumpimeta.h>
#include <string>
#include <vector>
#include <map>

#include <boost/lexical_cast.hpp>

namespace sstmac { namespace sw {


  /**
   * A refactored dumpi parser to read the newer binary-format dumpi
   * trace files.  Discards support for older-style ascii-based
   * files since nobody was using those except us anyway.
   *
   * TODO:  Fix this to read the metafile rather than the trace files directly
   * (would avoid hack needed where MPI world size is smaller than the total
   * cores allocated to the job).
   */
  class parsedumpi : public mpiapp {
  public:
    typedef boost::intrusive_ptr<parsedumpi> ptr;

  private:
    /// Private parts:
    class callbacks;

    /// The fileroot we plan to parse.
    std::string fileroot_;

    /// The time scaling factor.
    double timescaling_;
    
    /// The metafile object
    //dumpimeta::ptr dmeta_;

    std::string metafilename_;

    /// Convenience method.
    inline const mpicomm& world() {
      return mpi()->comm_world();
    }

  protected:
    /// Hi there.
    parsedumpi(softwareid id);



  public:
    /// Good news everyone!  We are getting a new parsedumpi.
    static parsedumpi::ptr
    construct(softwareid id);

    
    /// Return the dumpi trace file name
    /// @param rank the MPI rank of the desired trace
    /// @return the trace file name or an empty string if it does not exist
    //std::string dumpi_file_name(int rank);

    /// Wait!  That's not good news at all!
    virtual ~parsedumpi() throw();

    /// IT'S AN ADDAMS!
    virtual app::ptr clone(softwareid newid);

    virtual void
    consume_params(const std::list<std::string> &params);

    /// Return the number of processors
   // virtual int getnumprocs();


    /// Parse the tracefile.
    virtual void skeleton_main();
  };


} } // end of namespace sstmac

#endif // ! SSTMAC_SOFTWARE_SKELETONS_UNDUMPI_PARSEDUMPI_H_INCLUDED
