/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/threading/stackalloc.h>
#include <sstmac/software/threading/stackalloc-chunk.h>

#include <sstmac/common/errors.h>
#include <unistd.h>


namespace sstmac { namespace sw {
  //
  // Build.
  //
  stackalloc::stackalloc(size_t stacksize, size_t alloc_unit, bool use_mprot) :
    suggested_chunk_(alloc_unit), stacksize_(stacksize), use_mprot_(use_mprot)
  {
    size_t rem = stacksize_ % sysconf(_SC_PAGESIZE);
    if(rem) {
      stacksize_ += (sysconf(_SC_PAGESIZE) - rem);
    }
  }

  //
  // Goodbye.
  stackalloc::~stackalloc() {
  }

  //
  // Access the allocation size.
  //
  size_t stackalloc::stacksize() const {
    return stacksize_;
  }

  //
  // Get a stack memory region.
  //
  void* stackalloc::alloc() {
    if(available_.empty()) {
      //std::cerr << "stackalloc::alloc:  Allocating a new chunk\n";
      // grab a new chunk.
      void *buf = 0;
      while(! buf) {
        if(! chunk_.empty()) {
          buf = chunk_.back()->get_next_stack();
        }
        if(! buf) {
          chunk_.push_back
            (chunkptr_t(new chunk(stacksize_, suggested_chunk_, use_mprot_)));
        }
      }
      //std::cerr << "stackalloc::alloc:  buf==" << buf << "\n";
      return buf;
    }
    else {
      void *buf = available_.back();
      available_.pop_back();
      return buf;
    }
  }

  //
  // Return the given memory region.
  //
  void stackalloc::free(void* buf) {
    //allocated_t::iterator it = allocated_.find(buf);
    //if(it == allocated_.end()) {
    //  throw valueerror("stackalloc::free:  Buffer does not belong "
    //                   "to this allocator.");
    //}
    available_.push_back(buf);
    //allocated_.erase(it);
  }

  //
  // Clear all memory.
  //
  void stackalloc::clear() {
    chunk_.clear();
    available_.clear();
    //allocated_.clear();
  }


} } // end of namespace sstmac
