/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac_testutil.h>
#include <unistd.h> // getopt
#include <math.h> // pow

#include <stdlib.h>
#include <stdio.h>


static void
ringnet(double latency, long int bandwidth, int n1,
    interconnect::ptr &fab, topology::ptr &net)
{
  net =  ring::construct(n1);
  fab = circuit::construct(timestamp(latency), bandwidth, net);
}
static void
fullnet(double latency, long int bandwidth, interconnect::ptr &fab,
    topology::ptr &net)
{
  fab = fullnetwork::construct(timestamp(latency), bandwidth);
}


static void
print_help(const char *name)
{
  std::cerr << name << " [-h] [-d debug_string] [-n nodes] [-p ppn] [-i iterations] "
      << "[-a array_length] [-t]\n";
}

int
main(int argc, char **argv)
{
  bool helpflag = false, testflag = false;
  int opt;
  int nodes = 8, ppn = 1;
  int argc0 = argc;
  std::string dbgstr = "<debug> app";
  timestamp stat_window(10);
  //::testing::InitGoogleTest(&argc, argv);
  // If any googletest flags were found, then run google test.
  if (argc != argc0)
    {
      testflag = true;
    }
  while (!helpflag && (opt = getopt(argc, argv, "htd:n:p:s:")) != -1)
    {
      switch (opt)
        {
      case 'h':
        helpflag = true;
        break;
      case 't':
        testflag = true;
        break;
      case 'd':
        dbgstr = optarg;
        break;
      case 'n':
        nodes = atoi(optarg);
        break;
      case 'p':
        ppn = atoi(optarg);
        break;
      case 's':
        stat_window = timestamp(atof(optarg));
        break;

      default:
        std::cerr << "Unknown argument -" << opt << "\n";
        helpflag = true;
        }
    }
  if (helpflag)
    {
      print_help(argv[0]);
      return 0;
    }
  if (testflag)
    {
      //return RUN_ALL_TESTS();
    }
  bool errflag = false;
  if (nodes <= 0)
    {
      std::cerr << "Invalid node count \"" << nodes << "\"\n";
      errflag = true;
    }
  if (ppn <= 0)
    {
      std::cerr << "Invalid ppn count \"" << ppn << "\"\n";
      errflag = true;
    }


  if (errflag)
    {
      return 1;
    }
  debug::set_user_param(dbgstr);

  sstmac::sw::skeletonfactory::init_skeletons();

  sstmac::hw::interconnect::ptr fab;
  sstmac::hw::topology::ptr net;


   stat_time_window::TIME_INTERVAL = stat_window;

  std::cout << "-- testing mpi with fullnet" << std::endl;
  fullnet(1e-9, 125000000, fab, net);
  run_simple_sim(nodes, ppn, "MPI_testall", std::list<std::string>(), fab);


  std::cout << "-- testing mpi with ring" << std::endl;

  ringnet(1e-9, 125000000, nodes, fab, net);
  run_simple_sim(nodes, ppn, "MPI_testall", std::list<std::string>(), fab);

  return 0;
}
